@@ -20,7 +20,7 @@ use SQL::Abstract;
 use vars qw($VERSION @EXPORT $AUTHORITY);
 use base qw(Exporter);
 
-$VERSION   = '0.9_6';
+$VERSION   = '0.101';
 $AUTHORITY = 'cpan:SUKRIA';
 @EXPORT    = qw(has_p has_one has_many);
 
@@ -206,17 +206,20 @@ sub has_p {
 # TODO : later let the user override the bindings
 
 sub has_one {
-    my ($owned_class)   = @_;
-    my $class           = caller;
+    my ($name, %options) = @_;
+    my $class = caller;
 
+    my $owned_class       = $options{class_name} || $name;
     my $owned_table_name  = Coat::Persistent::Meta->table_name($owned_class);
     my $owned_primary_key = Coat::Persistent::Meta->primary_key($owned_class);
+    
+    my $attr_name = (defined $options{class_name}) ? $name : $owned_table_name ;
 
     # record the foreign key
     my $foreign_key = $owned_table_name . '_' . $owned_primary_key;
     has_p $foreign_key => ( isa => 'Int', '!caller' => $class );
 
-    my $symbol = "${class}::${owned_table_name}";
+    my $symbol = "${class}::${attr_name}";
     my $code   = sub {
         my ( $self, $object ) = @_;
 
@@ -237,6 +240,9 @@ sub has_one {
         }
     };
     _bind_code_to_symbol( $code, $symbol );
+
+    # save the accessor defined for that subobject
+    Coat::Persistent::Meta->accessor( $class => $attr_name );
 }
 
 # many relations means an instance of class A owns many instances
@@ -244,14 +250,20 @@ sub has_one {
 #     $a->bs returns B->find_by_a_id($a->id)
 # * B must provide a 'has_one A' statement for this to work
 sub has_many {
-    my ($owned_class)   = @_;
-    my $class           = caller;
-    
+    my ($name, %options)   = @_;
+    my $class = caller;
+
+    my $owned_class       = $options{class_name} || $name;
+
     # get the SQL table names and primary keys we need 
     my $table_name        = Coat::Persistent::Meta->table_name($class);
     my $primary_key       = Coat::Persistent::Meta->primary_key($class);
     my $owned_table_name  = Coat::Persistent::Meta->table_name($owned_class);
     my $owned_primary_key = Coat::Persistent::Meta->primary_key($owned_class);
+    
+    my $attr_name = (defined $options{class_name}) 
+                  ? $name 
+                  : $owned_table_name.'s' ;
 
     # FIXME : have to pluralize properly and let the user
     # disable the pluralisation.
@@ -269,18 +281,21 @@ sub has_many {
         else {
             foreach my $obj (@list) {
                 # is the object made of something appropriate?
-                confess "Not an object reference, expected $owned_class"
+                confess "Not an object reference, expected $owned_class, got ($obj)"
                   unless defined blessed $obj;
                 confess "Not an object of class $owned_class (got "
                   . blessed($obj) . ")"
                   unless blessed $obj eq $owned_class;
+                
                 # then set 
-                $obj->$table_name($self);
+                my $accessor = Coat::Persistent::Meta->accessor( $owned_class) || $table_name;
+                $obj->$accessor($self);
                 push @{ $self->{_subobjects} }, $obj;
             }
+            return scalar(@list) == scalar(@{$self->{_subobjects}});
         }
     };
-    _bind_code_to_symbol( $code, "${class}::${owned_table_name}s" );
+    _bind_code_to_symbol( $code, "${class}::${attr_name}" );
 }
 
 # When Coat::Persistent is imported, a couple of actions have to be 
@@ -291,9 +306,11 @@ sub import {
     my %options;
     %options = @stuff if @stuff % 2 == 0;
 
-    # Don't do our automagick inheritance if main is calling us
+    # Don't do our automagick inheritance if main is calling us or if the
+    # class has already been registered
     my $caller = caller;
     return if $caller eq 'main';
+    return if defined Coat::Persistent::Meta->registry( $class );
     
     # now, our caller inherits from Coat::Persistent
     eval { Coat::_extends_class( ['Coat::Persistent'], $caller ) };
@@ -421,6 +438,7 @@ sub find_by_sql {
             # create the object with attributes, and set virtual ones
             foreach my $r (@$rows) {
                 my $obj = $class->new(map { ($_ => $r->{$_}) } @given_attr);
+                $obj->init_on_find();
                 $obj->{$_} = $r->{$_} for @virtual_attr;
                 push @objects, $obj;
             }
@@ -442,6 +460,9 @@ sub find_by_sql {
 }
 
 
+sub init_on_find {
+}
+
 sub validate {
     my ($self, @args) = @_;
     my $class = ref($self);
@@ -520,7 +541,7 @@ sub save {
     my $dbh    = $class->dbh;
     my $table_name  = Coat::Persistent::Meta->table_name($class);
     my $primary_key = Coat::Persistent::Meta->primary_key($class);
-#    warn "save\n\ttable_name: $table_name\n\tprimary_key: $primary_key\n";
+    #warn "save\n\ttable_name: $table_name\n\tprimary_key: $primary_key\n";
 
     confess "Cannot save without a mapping defined for class " . ref $self
       unless defined $dbh;
@@ -554,6 +575,7 @@ sub save {
             $table_name, { %values, $primary_key => $self->$primary_key });
 
         # execute the query
+        #warn "sql: $sql ".join(', ', @values);
         my $sth = $dbh->prepare($sql);
         $sth->execute( @values )
           or confess "Unable to execute query \"$sql\" : $DBI::errstr";
@@ -577,6 +599,7 @@ sub save {
 # instance method & stuff
 sub _bind_code_to_symbol {
     my ( $code, $symbol ) = @_;
+
     {
         no strict 'refs';
         no warnings 'redefine', 'prototype';
